@@ -18,8 +18,9 @@
 #'   `c("conda-forge", <other channels>)`.
 #'
 #' @param packages A character vector, indicating package names which should be
-#'   installed or removed. Use `python=<version>` to request the installation
-#'   of a specific version of Python.
+#'   installed or removed. Use  \verb{<package>==<version>} to request the installation
+#'   of a specific version of a package. A `NULL` value for [conda_remove()] 
+#'   will be interpretted to `"--all"`, removing the entire environment.
 #'
 #' @param environment The path to an environment definition, generated via
 #'   (for example) [conda_export()], or via `conda env export`. When provided,
@@ -37,7 +38,7 @@
 #' @param file The path where the conda environment definition will be written.
 #'
 #' @param json Boolean; should the environment definition be written as JSON?
-#'   By default, conda exports environmentas as YAML.
+#'   By default, conda exports environments as YAML.
 #'
 #' @param pip Boolean; use `pip` for package installation? By default, packages
 #'   are installed from the active conda channels.
@@ -101,6 +102,7 @@ conda_list <- function(conda = "auto") {
 
   # resolve conda binary
   conda <- conda_binary(conda)
+  local_conda_paths(conda)
 
   # list envs -- discard stderr as Anaconda may emit warnings that can
   # otherwise be ignored; see e.g. https://github.com/rstudio/reticulate/issues/474
@@ -190,6 +192,7 @@ conda_create <- function(envname = NULL,
 {
   # resolve conda binary
   conda <- conda_binary(conda)
+  local_conda_paths(conda)
 
   # if environment is provided, use it directly
   if (!is.null(environment))
@@ -226,7 +229,7 @@ conda_create <- function(envname = NULL,
     args <- c(args, "-c", ch)
 
   # invoke conda
-  result <- system2(conda, shQuote(args))
+  result <- system2t(conda, shQuote(args))
   if (result != 0L) {
     fmt <- "Error creating conda environment '%s' [exit code %i]"
     stopf(fmt, envname, result, call. = FALSE)
@@ -252,7 +255,7 @@ conda_create_env <- function(envname, environment, conda) {
     "-f", shQuote(environment)
   )
 
-  result <- system2(conda, args)
+  result <- system2t(conda, args)
   if (result != 0L) {
     fmt <- "Error creating conda environment [exit code %i]"
     stopf(fmt, result)
@@ -273,6 +276,7 @@ conda_clone <- function(envname, ..., clone = "base", conda = "auto") {
 
   # resolve conda binary
   conda <- conda_binary(conda)
+  local_conda_paths(conda)
 
   # resolve environment name
   envname <- condaenv_resolve(envname)
@@ -287,7 +291,7 @@ conda_clone <- function(envname, ..., clone = "base", conda = "auto") {
   args <- c(args, "--clone", clone)
 
   # invoke conda
-  result <- system2(conda, shQuote(args))
+  result <- system2t(conda, shQuote(args))
   if (result != 0L) {
     fmt <- "Error creating conda environment '%s' [exit code %i]"
     stopf(fmt, envname, result, call. = FALSE)
@@ -312,6 +316,7 @@ conda_export <- function(envname,
 {
   # resolve parameters
   conda <- conda_binary(conda)
+  local_conda_paths(conda)
   envname <- condaenv_resolve(envname)
 
   # build conda argument list,
@@ -349,6 +354,7 @@ conda_remove <- function(envname,
 {
   # resolve conda binary
   conda <- conda_binary(conda)
+  local_conda_paths(conda)
 
   # resolve environment name
   envname <- condaenv_resolve(envname)
@@ -357,9 +363,9 @@ conda_remove <- function(envname,
   if (is.null(packages))
     packages <- "--all"
 
-  # remove packges (or the entire environment)
+  # remove packages (or the entire environment)
   args <- conda_args("remove", envname, packages)
-  result <- system2(conda, shQuote(args))
+  result <- system2t(conda, shQuote(args))
   if (result != 0L) {
     stop("Error ", result, " occurred removing conda environment ", envname,
          call. = FALSE)
@@ -400,6 +406,7 @@ conda_install <- function(envname = NULL,
 
   # resolve conda binary
   conda <- conda_binary(conda)
+  local_conda_paths(conda)
 
   # resolve environment name
   envname <- condaenv_resolve(envname)
@@ -439,7 +446,7 @@ conda_install <- function(envname = NULL,
   # (should be no-op if that copy of Python already installed)
   if (!is.null(python_version)) {
     args <- conda_args("install", envname, python_package)
-    status <- system2(conda, shQuote(args))
+    status <- system2t(conda, shQuote(args))
     if (status != 0L) {
       fmt <- "installation of '%s' into environment '%s' failed [error code %i]"
       msg <- sprintf(fmt, python_package, envname, status)
@@ -476,7 +483,7 @@ conda_install <- function(envname = NULL,
     args <- c(args, "-c", ch)
 
   args <- c(args, python_package, packages)
-  result <- system2(conda, shQuote(args))
+  result <- system2t(conda, shQuote(args))
 
   # check for errors
   if (result != 0L) {
@@ -501,8 +508,8 @@ conda_binary <- function(conda = "auto") {
   }
 
   conda <- normalizePath(conda, winslash = "/", mustWork = FALSE)
-  if (!grepl("^conda", basename(conda)))
-    stop("Supplied path is not a conda binary: ", sQuote(conda))
+  if (!grepl("^(conda|mamba)", basename(conda)))
+    warning("Supplied path is not a conda binary: ", sQuote(conda))
 
   # if the user has requested a conda binary in the 'condabin' folder,
   # try to find and use its sibling in the 'bin' folder instead as
@@ -538,7 +545,11 @@ conda_exe <- conda_binary
 #' @export
 conda_version <- function(conda = "auto") {
   conda_bin <- conda_binary(conda)
-  system2(conda_bin, "--version", stdout = TRUE)
+  local_conda_paths(conda_bin)
+  out <- system2(conda_bin, "--version", stdout = TRUE)
+
+  # mamba --version gives multi-line output, with the conda version on the last line.
+  tail(out, 1)
 }
 
 
@@ -548,6 +559,7 @@ conda_update <- function(conda = "auto") {
 
   # resolve conda
   conda <- conda_binary(conda)
+  local_conda_paths(conda)
 
   # compute base path
   prefix <- system2(conda, c("info", "--base"), stdout = TRUE)
@@ -558,8 +570,7 @@ conda_update <- function(conda = "auto") {
   name <- if ("anaconda" %in% envlist$name) "anaconda" else "conda"
 
   # attempt update
-  system2(conda, c("update", "--prefix", shQuote(prefix), "--yes", name))
-
+  system2t(conda, c("update", "--prefix", shQuote(prefix), "--yes", name))
 }
 
 numeric_conda_version <- function(conda = "auto", version_string = conda_version(conda)) {
@@ -753,6 +764,7 @@ is_condaenv <- function(dir) {
 conda_list_packages <- function(envname = NULL, conda = "auto", no_pip = TRUE) {
 
   conda <- conda_binary(conda)
+  local_conda_paths(conda)
   envname <- condaenv_resolve(envname)
 
   # create the environment
@@ -798,8 +810,12 @@ conda_installed <- function() {
 
 conda_run <- function(cmd, args = c(), conda = "auto", envname = NULL,
                       run_args = c("--no-capture-output"), ...) {
+  # `conda run` is very unreliable, best avoided. known issues:
+  #  - fails if user doesn't have write permissions to conda installation
+  #  - fails if arguments need to be quoted
 
   conda <- conda_binary(conda)
+  local_conda_paths(conda)
   envname <- condaenv_resolve(envname)
 
   if (numeric_conda_version(conda) < "4.9")
@@ -813,7 +829,7 @@ Run `miniconda_update('%s')` to update conda.", conda)
   else
     in_env <- c("--name", envname)
 
-  system2(conda, c("run", in_env, run_args,
+  system2t(conda, c("run", in_env, run_args,
                    shQuote(cmd), args), ...)
 }
 
@@ -828,8 +844,10 @@ conda_run2 <- function(...) {
 
 conda_run2_windows <-
   function(cmd, args = c(), conda = "auto", envname = NULL,
-           cmd_line = paste(shQuote(cmd), paste(args, collapse = " "))) {
+           cmd_line = paste(shQuote(cmd), paste(args, collapse = " ")),
+           intern = FALSE, echo = !intern) {
   conda <- normalizePath(conda_binary(conda))
+  local_conda_paths(conda)
 
   if (identical(envname, "base"))
     envname <- file.path(dirname(conda), "../..")
@@ -839,20 +857,33 @@ conda_run2_windows <-
   if (grepl("[/\\]", envname))
     envname <- normalizePath(envname)
 
+  activate.bat <- normalizePath(file.path(dirname(conda), "activate.bat"),
+                                mustWork = FALSE)
+
+  activate_cmd <-
+    if (file.exists(activate.bat)) {
+      paste("CALL", shQuote(activate.bat), shQuote(envname))
+    } else {
+      paste("CALL", shQuote(conda), "activate", shQuote(envname))
+    }
+
   fi <- tempfile(fileext = ".bat")
   on.exit(unlink(fi))
   writeLines(c(
-    paste("CALL", shQuote(conda), "activate", shQuote(envname)),
+    if (!echo) "@echo off",
+    activate_cmd,
     cmd_line
   ), fi)
 
-  shell(fi)
+  shell(fi, intern = intern)
 }
 
 conda_run2_nix <-
   function(cmd, args = c(), conda = "auto", envname = NULL,
-           cmd_line = paste(shQuote(cmd), paste(args, collapse = " "))) {
+           cmd_line = paste(shQuote(cmd), paste(args, collapse = " ")),
+           intern = FALSE, echo = !intern) {
   conda <- normalizePath(conda_binary(conda))
+  local_conda_paths(conda)
   activate <- normalizePath(file.path(dirname(conda), "activate"))
 
   if (!identical(envname, "base")) {
@@ -863,14 +894,24 @@ conda_run2_nix <-
 
   fi <- tempfile(fileext = ".sh")
   on.exit(unlink(fi))
-  writeLines(c(
+
+  commands <- c(
     paste(".", activate),
     if (!identical(envname, "base"))
       paste("conda activate", shQuote(envname)),
-    'echo "Activated conda python: $(which python)"',
     cmd_line
-  ), fi)
-  system2(Sys.which("sh"), fi)
+  )
+
+  # set -x is too verbose, includes all the commands made by conda scripts
+  # so we manually echo the top-level commands only
+  if (echo)
+    commands <- as.vector(rbind(
+      paste("echo", shQuote(paste("+", commands))),
+      commands))
+
+  writeLines(commands, fi)
+  system2(Sys.which("bash"), fi,
+          stdout = if (identical(intern, FALSE)) "" else intern)
 }
 
 
@@ -917,3 +958,39 @@ get_python_conda_info <- function(python) {
   )
 
 }
+
+
+conda_prefix <- function(conda = "auto") {
+  conda <- normalizePath(conda_binary(conda),
+                         winslash = "/", mustWork = TRUE)
+  if(!is_windows())
+    # PREFIX/bin/conda
+    return(dirname(dirname(conda)))
+
+  # on Windows, conda can be a few places:
+  ## PREFIX/Scripts/conda.exe
+  ## PREFIX/condabin/conda.bat
+  ## PREFIX/conda.exe
+  ## others? maybe under mingw-32/ or Library/ or Tools/?
+  ## we punt and just ask conda
+  system2(conda, c("info", "--base"), stdout = TRUE)
+}
+
+conda_bin_paths <- function(conda = "auto") {
+  prefix <- conda_prefix(conda)
+  paths <- file.path(prefix, c(
+    "condabin",
+    "bin",
+    "Scripts",
+    "Library/bin",
+    "Library/usr/bin",
+    "Library/mingw-w64/bin"
+  ))
+  paths[dir.exists(paths)]
+}
+
+
+local_conda_paths <- function(conda, action = "prefix",
+                              .local_envir = parent.frame()) {
+  withr::local_path(conda_bin_paths(conda), action, .local_envir)
+}
