@@ -322,6 +322,10 @@ py_discover_config <- function(required_module = NULL, use_environment = NULL) {
   if (nzchar(python))
     python_versions <- c(python_versions, python)
 
+  python <- as.character(Sys.which("python"))
+  if (nzchar(python))
+    python_versions <- c(python_versions, python)
+
   # provide other common locations
   python_versions <- c(python_versions, py_discover_config_fallbacks())
 
@@ -343,6 +347,19 @@ py_discover_config <- function(required_module = NULL, use_environment = NULL) {
     python_versions <- python_versions[size != 0]
   }
 
+  # remove msys2 / cygwin python executables.
+  # path translation going to and from msys2 currently not implemented.
+  # E.g.: "C:\foo\bar" -> "/c/foo/bar" and  "/foo/bar" -> "C:\rtools43\foo\bar"
+  # https://github.com/rstudio/reticulate/issues/1325
+  if (is_windows()) {
+    python_sys_platforms <- vapply(
+      python_versions, system2, "",
+      args = c("-c", shQuote("import sys; print(sys.platform)")),
+      stdout = TRUE)
+
+    python_versions <- python_versions[python_sys_platforms != "cygwin"]
+  }
+
   # scan until we find a version of python that meets our qualifying conditions
   valid_python_versions <- c()
   for (python_version in python_versions) {
@@ -561,21 +578,19 @@ python_munge_path <- function(python) {
   if (is_conda_python(python)) {
     conda_info <- get_python_conda_info(python)
 
-    if (numeric_conda_version(conda_info$conda) >= "4.9") {
-
-      new_path <- conda_run(
-        "python",
-        c("-c", shQuote("import os; print(os.environ['PATH'])")),
-        conda = conda_info$conda,
-        envname = conda_info$root,
-        stdout = TRUE
-      )
-
-      old_path <- Sys.getenv("PATH")
-      Sys.setenv("PATH" = new_path)
-      return(old_path)
-    }
+    new_path <- conda_run2(
+      "python",
+      c("-c", shQuote("import os; print(os.environ['PATH'])")),
+      conda = conda_info$conda,
+      envname = conda_info$root,
+      intern = TRUE
+    )
+    # maybe discard unsilenceable warnings, see issue #1303
+    new_path <- new_path[length(new_path)]
 
+    old_path <- Sys.getenv("PATH")
+    Sys.setenv("PATH" = new_path)
+    return(old_path)
   }
 
   if (is_windows()) {
@@ -959,7 +974,8 @@ normalize_python_path <- function(python) {
       python <- file.path(python, "python")
 
     # append .exe if necessary on windows
-    if (is_windows() && (!grepl("^.*\\.exe$", tolower(python))))
+    # accept .bat for pyenv-win shim
+    if (is_windows() && (!grepl("^.*\\.(exe|bat)$", tolower(python))))
       python <- paste0(python, ".exe")
 
     # return
