@@ -1,7 +1,8 @@
 
 #' @export
 print.python.builtin.object <- function(x, ...) {
-  str(x, ...)
+  writeLines(py_repr(x))
+  invisible(x)
 }
 
 
@@ -34,7 +35,7 @@ as.character.python.builtin.object <- function(x, ...) {
 #'
 #' @param encoding Encoding to use for conversion (defaults to utf-8)
 #' @param errors Policy for handling conversion errors. Default is 'strict'
-#'  which raises an error. Other possible values are 'ignore' and 'replace'
+#'  which raises an error. Other possible values are 'ignore' and 'replace'.
 #'
 #' @export
 as.character.python.builtin.bytes <- function(x, encoding = "utf-8", errors = "strict", ...) {
@@ -286,7 +287,7 @@ as.environment.python.builtin.object <- function(x) {
     names <- sort(names, decreasing = FALSE)
 
     # get the types
-    types <- py_suppress_warnings(py_get_attribute_types(x, names))
+    types <- py_suppress_warnings(py_get_attr_types(x, names))
   }
 
 
@@ -375,7 +376,7 @@ plot.numpy.ndarray <- function(x, y, ...) {
 #' @return A Python dictionary
 #'
 #' @note The returned dictionary will not automatically convert its elements
-#'   from Python to R. You can do manual converstion with the [py_to_r()]
+#'   from Python to R. You can do manual conversion with the [py_to_r()]
 #'   function or pass `convert = TRUE` to request automatic conversion.
 #'
 #' @export
@@ -438,7 +439,7 @@ py_dict <- function(keys, values, convert = FALSE) {
 #'
 #' @return A Python tuple
 #' @note The returned tuple will not automatically convert its elements from
-#'   Python to R. You can do manual converstion with the [py_to_r()] function or
+#'   Python to R. You can do manual conversion with the [py_to_r()] function or
 #'   pass `convert = TRUE` to request automatic conversion.
 #'
 #' @export
@@ -478,24 +479,76 @@ length.python.builtin.tuple <- function(x) {
 
 #' Length of Python object
 #'
-#' Get the length of a Python object (equivalent to the Python `len()`
-#' built in function).
+#' Get the length of a Python object. This is equivalent to calling
+#' the Python builtin `len()` function on the object.
 #'
-#' @param x Python object
+#' Not all Python objects have a defined length. For objects without a defined
+#' length, calling `py_len()` will throw an error. If you'd like to instead
+#' infer a default length in such cases, you can set the `default` argument
+#' to e.g. `1L`, to treat Python objects without a `__len__` method as having
+#' length one.
 #'
-#' @return Length as integer
+#' @param x A Python object.
+#'
+#' @param default The default length value to return, in the case that
+#'   the associated Python object has no `__len__` method. When `NULL`
+#'   (the default), an error is emitted instead.
+#'
+#' @return The length of the object, as a numeric value.
 #'
 #' @export
-py_len <- function(x) {
+py_len <- function(x, default = NULL) {
+
+  # return 0 if Python not yet available
   if (py_is_null_xptr(x) || !py_available())
-    0L
-  else
-    as_r_value(x$`__len__`())
+    return(0L)
+
+  # delegate to C++
+  py_len_impl(x, default)
 }
 
 #' @export
 length.python.builtin.list <- function(x) {
-  py_len(x)
+  py_list_length(x)
+}
+
+#' @export
+length.python.builtin.object <- function(x) {
+
+  # return 0 if Python not yet available
+  if (py_is_null_xptr(x) || !py_available())
+    return(0L)
+
+  # otherwise, try to invoke the object's __len__ method
+  n <- py_len_impl(x, NA_integer_)
+  if (is.na(n))
+    # if the object didn't have a __len__ method, or __len__ raised an
+    # Exception, try instead to invoke its __bool__ method
+    return(as.integer(py_bool_impl(x)))
+
+  n
+}
+
+
+#' Python Truthiness
+#'
+#' Equivalent to `bool(x)` in Python, or `not not x`.
+#'
+#' If the Python object defines a `__bool__` method, then that is invoked.
+#' Otherwise, if the object defines a `__len__` method, then `TRUE` is
+#' returned if the length is nonzero. If neither `__len__` nor `__bool__`
+#' are defined, then the Python object is considered `TRUE`. If `x`
+#'
+#' @param x, A python object.
+#'
+#' @return An R scalar logical: `TRUE` or `FALSE`. If `x` is a
+#'   null pointer or Python is not initialized, `FALSE` is returned.
+#' @export
+py_bool <- function(x) {
+  if (py_is_null_xptr(x) || !py_available())
+    FALSE
+  else
+    py_bool_impl(x)
 }
 
 
@@ -508,7 +561,7 @@ length.python.builtin.list <- function(x) {
 #'   they are 8-bit string objects. This function enables you to
 #'   obtain a Python unicode object from an R character vector
 #'   when running under Python 2 (under Python 3 a standard Python
-#'   string object is returend).
+#'   string object is returned).
 #'
 #' @export
 py_unicode <- function(str) {
@@ -643,15 +696,15 @@ iterate <- function(it, f = base::identity, simplify = TRUE) {
 #' @rdname iterate
 #' @export
 iter_next <- function(it, completed = NULL) {
-  
+
   # TODO: would like to use PyIter_Check() but that is only implemented
   # as a macro in Python 2.x and requires copying more headers
   iterable <- py_has_attr(it, "__next__") || py_has_attr(it, "next")
   if (!iterable)
     stop("object is not iterable", call. = FALSE)
-  
+
   py_iter_next(it, completed)
-  
+
 }
 
 
@@ -732,9 +785,9 @@ py_set_attr <- function(x, name, value) {
 }
 
 #' The Python None object
-#' 
+#'
 #' Get a reference to the Python `None` object.
-#' 
+#'
 #' @export
 py_none <- function() {
   ensure_python_initialized()
@@ -742,10 +795,10 @@ py_none <- function() {
 }
 
 #' Delete an attribute of a Python object
-#' 
+#'
 #' @param x A Python object.
 #' @param name The attribute name.
-#' 
+#'
 #' @export
 py_del_attr <- function(x, name) {
   ensure_python_initialized()
@@ -770,7 +823,16 @@ py_list_attributes <- function(x) {
   attrs
 }
 
+py_get_attr_types <- function(x,
+                              names,
+                              resolve_properties = FALSE)
+{
+  ensure_python_initialized()
+  if (py_is_module_proxy(x))
+    py_resolve_module_proxy(x)
 
+  py_get_attr_types_impl(x, names, resolve_properties)
+}
 
 #' Get an item from a Python object
 #'
@@ -791,14 +853,14 @@ py_get_item <- function(x, key, silent = FALSE) {
   ensure_python_initialized()
   if (py_is_module_proxy(x))
     py_resolve_module_proxy(x)
- 
-  # NOTE: for backwards compatibility, we make sure to return an R NULL on error 
+
+  # NOTE: for backwards compatibility, we make sure to return an R NULL on error
   if (silent) {
     tryCatch(py_get_item_impl(x, key, FALSE), error = function(e) NULL)
   } else {
     py_get_item_impl(x, key, FALSE)
   }
-  
+
 }
 
 #' Set an item for a Python object
@@ -851,29 +913,22 @@ py_del_item <- function(x, name) {
 
 
 
-#' Unique identifer for Python object
-#'
-#' Get a globally unique identifer for a Python object.
-#'
-#' @note In the current implementation of CPython this is the
-#'  memory address of the object.
-#'
-#' @param object Python object
-#'
-#' @return Unique identifer (as integer) or `NULL`
-#'
-#' @export
-py_id <- function(object) {
-  if (py_is_null_xptr(object) || !py_available())
-    NULL
-  else {
-    py <- import_builtins()
-    py$id(object)
-  }
-}
 
 
-#' An S3 method for getting the string representation of a Python object
+#' String representation of a python object.
+#'
+#' This is equivalent to calling `str(object)` or `repr(object)` in Python.
+#'
+#' In Python, calling `print()` invokes the builtin `str()`, while auto-printing
+#' an object at the REPL invokes the builtin `repr()`.
+#'
+#' In \R, the default print method for python objects invokes `py_repr()`, and
+#' the default `format()` and `as.character()` methods invoke `py_str()`.
+#'
+#' For historical reasons, `py_str()` is also an \R S3 method that allows R
+#' authors to customize the the string representation of a Python object from R.
+#' New code is recommended to provide a `format()` and/or `print()` S3 R method
+#' for python objects instead.
 #'
 #' @param object Python object
 #' @param ... Unused
@@ -885,7 +940,7 @@ py_id <- function(object) {
 #' @export
 py_str <- function(object, ...) {
   if (!inherits(object, "python.builtin.object"))
-    py_str.default(object)
+    "<not a python object>"
   else if (py_is_null_xptr(object) || !py_available())
     "<pointer: 0x0>"
   else
@@ -897,12 +952,20 @@ py_str.default <- function(object, ...) {
   "<not a python object>"
 }
 
-
 #' @export
 py_str.python.builtin.object <- function(object, ...) {
+  py_str_impl(object)
+}
 
-  # get default rep
-  str <- py_str_impl(object)
+
+#' @export
+format.python.builtin.object <- function(x, ...) {
+
+  if (py_is_null_xptr(x) || !py_available())
+    return("<pointer: 0x0>")
+
+  # get default rep, potentially user defined S3
+  str <- py_str(x)
 
   # remove e.g. 'object at 0x10d084710'
   str <- gsub(" object at 0x\\w{4,}", "", str)
@@ -913,8 +976,7 @@ py_str.python.builtin.object <- function(object, ...) {
 
 #' @export
 py_str.python.builtin.bytearray <- function(object, ...) {
-  builtins <- import_builtins()
-  paste0("python.builtin.bytearray (", builtins$len(object), " bytes)")
+  paste0("python.builtin.bytearray (", py_len_impl(object), " bytes)")
 }
 
 #' @export
@@ -938,20 +1000,30 @@ py_str.python.builtin.tuple <- function(object, ...) {
 }
 
 py_collection_str <- function(name, object) {
-  len <- py_collection_len(object)
+  len <- py_len_impl(object)
   if (len > 10)
     paste0(name, " (", len, " items)")
   else
     py_str.python.builtin.object(object)
 }
 
-py_collection_len <- function(object) {
-  # do this dance so we can call __len__ on dictionaries (which
-  # otherwise overload the $)
-  len <- py_get_attr(object, "__len__")
-  py_to_r(py_call(len))
+.print.via.format <- function(x, ...) {
+  writeLines(format(x, ...))
+  invisible(x)
 }
 
+#' @export
+print.python.builtin.bytearray <- .print.via.format
+#' @export
+print.python.builtin.tuple <- .print.via.format
+#' @export
+print.python.builtin.module <- .print.via.format
+#' @export
+print.python.builtin.list <- .print.via.format
+#' @export
+print.python.builtin.dict <- .print.via.format
+
+
 #' Suppress Python warnings for an expression
 #'
 #' @param expr Expression to suppress warnings for
@@ -1024,7 +1096,7 @@ py_capture_output <- function(expr, type = c("stdout", "stderr")) {
 
   # get output tools helper functions
   output_tools <- import("rpytools.output")
-  
+
   # scope output capture
   capture_stdout <- "stdout" %in% type
   capture_stderr <- "stderr" %in% type
@@ -1036,7 +1108,7 @@ py_capture_output <- function(expr, type = c("stdout", "stderr")) {
 
   # collect output
   output_tools$collect_output()
-  
+
 }
 
 py_flush_output <- function() {
@@ -1045,10 +1117,10 @@ py_flush_output <- function() {
     return()
 
   sys <- import("sys", convert = TRUE)
-  
+
   if (!is.null(sys$stdout) && is.function(sys$stdout$flush))
     sys$stdout$flush()
-  
+
   if (!is.null(sys$stderr) && is.function(sys$stderr$flush))
     sys$stderr$flush()
 
@@ -1058,17 +1130,20 @@ py_flush_output <- function() {
 
 #' Run Python code
 #'
-#' Execute code within the the \code{__main__} Python module.
+#' Execute code within the scope of the \code{__main__} Python module.
 #'
 #' @inheritParams import
-#' @param code Code to execute
-#' @param file Source file
-#' @param local Whether to create objects in a local/private namespace (if
-#'   `FALSE`, objects are created within the main module).
 #'
-#' @return For `py_eval()`, the result of evaluating the expression; For
-#'   `py_run_string()` and `py_run_file()`, the dictionary associated with
-#'   the code execution.
+#' @param code The Python code to be executed.
+#' @param file The Python script to be executed.
+#' @param local Boolean; should Python objects be created as part of
+#'   a local / private dictionary? If `FALSE`, objects will be created within
+#'   the scope of the Python main module.
+#'
+#' @return A Python dictionary of objects. When `local` is `FALSE`, this
+#'   dictionary captures the state of the Python main module after running
+#'   the provided code. Otherwise, only the variables defined and used are
+#'   captured.
 #'
 #' @name py_run
 #'
@@ -1083,11 +1158,46 @@ py_run_string <- function(code, local = FALSE, convert = TRUE) {
 #' @export
 py_run_file <- function(file, local = FALSE, convert = TRUE) {
   ensure_python_initialized()
-  on.exit(py_flush_output(), add = TRUE)
-  invisible(py_run_file_impl(file, local, convert))
+  invisible(py_run_file_impl(path.expand(file), local, convert))
 }
 
-#' @rdname py_run
+#' Evaluate a Python Expression
+#'
+#' Evaluate a single Python expression, in a way analogous to the Python
+#' `eval()` built-in function.
+#'
+#' @param code A single Python expression.
+#' @param convert Boolean; automatically convert Python objects to R?
+#'
+#' @return The result produced by evaluating `code`, converted to an `R`
+#'   object when `convert` is set to `TRUE`.
+#'
+#' @section Caveats:
+#'
+#' `py_eval()` only supports evaluation of 'simple' Python expressions.
+#' Other expressions (e.g. assignments) will fail; e.g.
+#'
+#' ```
+#' > py_eval("x = 1")
+#' Error in py_eval_impl(code, convert) :
+#'   SyntaxError: invalid syntax (reticulate_eval, line 1)
+#' ```
+#'
+#' and this mirrors what one would see in a regular Python interpreter:
+#'
+#' ```
+#' >>> eval("x = 1")
+#' Traceback (most recent call last):
+#'   File "<stdin>", line 1, in <module>
+#'   File "<string>", line 1
+#' x = 1
+#' ^
+#'   SyntaxError: invalid syntax
+#' ```
+#'
+#' The [py_run_string()] method can be used if the evaluation of arbitrary
+#' Python code is required.
+#'
 #' @export
 py_eval <- function(code, convert = TRUE) {
   ensure_python_initialized()
@@ -1095,7 +1205,7 @@ py_eval <- function(code, convert = TRUE) {
 }
 
 #' The builtin constant Ellipsis
-#' 
+#'
 #' @export
 py_ellipsis <- function() {
   builtins <- import_builtins(convert = FALSE)
@@ -1103,46 +1213,46 @@ py_ellipsis <- function() {
 }
 
 py_callable_as_function <- function(callable, convert) {
-  
+
   force(callable)
   force(convert)
-  
+
   function(...) {
-    
+
     dots <- py_resolve_dots(list(...))
     result <- py_call_impl(callable, dots$args, dots$keywords)
-    
+
     if (convert)
       result <- py_to_r(result)
-      
+
     if (is.null(result))
       invisible(result)
     else
       result
-    
+
   }
-  
+
 }
 
 py_resolve_formals <- function(callback) {
-  
+
   object <- attr(callback, "py_object")
   if (!inherits(object, "python.builtin.object"))
     return(NULL)
-  
+
   tryCatch(py_get_formals(object), error = function(e) NULL)
-  
+
 }
 
 py_resolve_dots <- function(dots) {
-  
+
   nms <- names(dots)
   if (is.null(nms))
     return(list(args = dots, keywords = list()))
-  
+
   named <- nzchar(nms)
   list(args = dots[!named], keywords = dots[named])
-  
+
 }
 
 
@@ -1180,7 +1290,7 @@ py_resolve_module_proxy <- function(proxy) {
   before_load <- collect_value("before_load")
   on_load <- collect_value("on_load")
   on_error <- collect_value("on_error")
-  
+
   # execute before load handler
   if (is.function(before_load))
     before_load()
@@ -1239,13 +1349,13 @@ py_filter_classes <- function(classes) {
   classes
 }
 
-py_inject_r <- function(envir) {
+py_inject_r <- function() {
 
   # don't inject 'r' if there's already an 'r' object defined
   main <- import_main(convert = FALSE)
   if (py_has_attr(main, "r"))
     return(FALSE)
-  
+
   # define our 'R' class
   py_run_string("class R(object): pass")
 
@@ -1253,20 +1363,18 @@ py_inject_r <- function(envir) {
   main <- import_main(convert = FALSE)
   R <- main$R
 
-  # extract active knit environment
-  if (is.null(envir)) {
-    .knitEnv <- yoink("knitr", ".knitEnv")
-    envir <- .knitEnv$knit_global
-  }
-
   # define the getters, setters we'll attach to the Python class
   getter <- function(self, code) {
+    envir <- py_resolve_envir()
     object <- eval(parse(text = as_r_value(code)), envir = envir)
     r_to_py(object, convert = is.function(object))
   }
 
   setter <- function(self, name, value) {
-    envir[[as_r_value(name)]] <<- as_r_value(value)
+    envir <- py_resolve_envir()
+    name  <- as_r_value(name)
+    value <- as_r_value(value)
+    assign(name, value, envir = envir)
   }
 
   py_set_attr(R, "__getattr__", getter)
@@ -1276,57 +1384,210 @@ py_inject_r <- function(envir) {
 
   # now define the R object
   py_run_string("r = R()")
-  
+
   # remove the 'R' class object
   py_del_attr(main, "R")
-  
+
   # indicate success
   TRUE
 
 }
 
+py_resolve_envir <- function() {
+
+  # if an environment has been set, use it
+  envir <- getOption("reticulate.engine.environment")
+  if (is.environment(envir))
+    return(envir)
+
+  # if we're running in a knitr document, use the knit env
+  if ("knitr" %in% loadedNamespaces()) {
+    .knitEnv <- yoink("knitr", ".knitEnv")
+    envir <- .knitEnv$knit_global
+    if (is.environment(envir))
+      return(envir)
+  }
+
+  # if we're running in a testthat test, use the rlang reported envir
+  envir <- getOption("rlang_trace_top_env")
+  if (is.environment(envir))
+    return(envir)
+
+  # otherwise, default to the global environment
+  envir %||% globalenv()
+
+}
+
 py_inject_hooks <- function() {
-  
+
   builtins <- import_builtins(convert = TRUE)
-  
+
   input <- function(prompt = "") {
-    
+
     response <- tryCatch(
       readline(prompt),
       interrupt = identity
     )
-    
+
     if (inherits(response, "interrupt"))
       stop("KeyboardInterrupt", call. = FALSE)
-    
+
     r_to_py(response)
-    
+
   }
-  
+
   # override input function
-  if (interactive()) {
+  if (interactive() && was_python_initialized_by_reticulate()) {
     name <- if (is_python3()) "input" else "raw_input"
     builtins[[name]] <- input
   }
-  
+
   # register module import callback
-  useImportHook <- getOption("reticulate.useImportHook", default = TRUE)
+  useImportHook <- getOption("reticulate.useImportHook", default = is_python3())
   if (useImportHook) {
-    loader <- import("rpytools.loader")
-    loader$initialize(py_module_loaded)
+    loader <- import("rpytools.loader", convert = TRUE)
+    loader$initialize(py_module_onload)
   }
-  
+
 }
 
-py_module_loaded <- function(module) {
-  
+py_module_onload <- function(module) {
+
+  # log module loading if requested
+  if (getOption("reticulate.logModuleLoad", default = FALSE)) {
+    writeLines(sprintf("Loaded module '%s'", module))
+  }
+
   # retrieve and clear list of hooks
   hookName <- paste("reticulate", module, "load", sep = "::")
   hooks <- getHook(hookName)
   setHook(hookName, NULL, action = "replace")
-  
+
   # run hooks
   for (hook in hooks)
     tryCatch(hook(), error = warning)
-  
+
+}
+
+py_module_loaded <- function(module) {
+  sys <- import("sys", convert = TRUE)
+  modules <- sys$modules
+  module %in% names(modules)
+}
+
+py_register_load_hook <- function(module, hook) {
+
+  # if the module is already loaded, just run the hook
+  if (py_module_loaded(module))
+    return(hook())
+
+  # otherwise, register the hook to be run on next load
+  name <- paste("reticulate", module, "load", sep = "::")
+  setHook(name, hook)
+
+}
+
+py_set_interrupt <- function() {
+  py_set_interrupt_impl()
+}
+
+#' @export
+format.python.builtin.traceback <- function(x, ..., limit = NULL) {
+  import("traceback")$format_tb(x, limit)
+}
+
+
+#' @rdname py_last_error
+#' @export
+py_clear_last_error <- function() {
+  py_last_error(NULL)
+}
+
+#' Get or (re)set the last Python error encountered.
+#'
+#' @param exception A python exception object. If provided, the provided
+#'   exception is set as the last exception.
+#'
+#' @return For `py_last_error()`, `NULL` if no error has yet been encountered.
+#'   Otherwise, a named list with entries:
+#'
+#'   +  `"type"`: R string, name of the exception class.
+#'
+#'   +  `"value"`: R string, formatted exception message.
+#'
+#'   +  `"traceback"`: R character vector, the formatted python traceback,
+#'
+#'   +  `"message"`: The full formatted raised exception, as it would be printed in
+#'   Python. Includes the traceback, type, and value.
+#'
+#' And attribute `"exception"`, a `'python.builtin.Exception'` object.
+#'
+#' The named list has `class` `"py_error"`, and has a default `print` method
+#' that is the equivalent of `cat(py_last_error()$message)`.
+#'
+#' @examples
+#' \dontrun{
+#' # run python code that might error,
+#' # without modifying the user-visible python exception
+#'
+#' safe_len <- function(x) {
+#'   last_err <- py_last_error()
+#'   tryCatch({
+#'     # this might raise a python exception if x has no `__len__` method.
+#'     import_builtins()$len(x)
+#'   }, error = function(e) {
+#'     # py_last_error() was overwritten, is now "no len method for 'object'"
+#'     py_last_error(last_err) # restore previous exception
+#'     -1L
+#'   })
+#' }
+#'
+#' safe_len(py_eval("object"))
+#' }
+#'
+#' @export
+py_last_error <- function(exception) {
+  if (!missing(exception)) {
+    # set as the last exception
+    if (inherits(exception, "py_error"))
+      exception <- attr(exception, "exception", TRUE)
+
+    if (!is.null(exception) &&
+        !inherits(exception, "python.builtin.Exception"))
+      stop("`exception` must be NULL, a `py_error`, or a 'python.builtin.Exception'")
+
+    on.exit(.globals$py_last_exception <- exception)
+    return(invisible(.globals$py_last_exception))
+  }
+
+  e <- .globals$py_last_exception
+
+  if (is.null(e))
+    return(NULL)
+
+  if (!py_available() || py_is_null_xptr(e)) {
+    .globals$py_last_exception <- NULL
+    return(NULL)
+  }
+
+  etype <- py_get_attr_impl(e, "__class__")
+  etb <- py_get_attr_impl(e, "__traceback__", TRUE)
+  traceback <- import("traceback")
+
+  out <- list(
+    type = py_get_attr_impl(etype, "__name__", TRUE),
+    value = py_str_impl(e),
+    traceback = if (!is.null(etb)) traceback$format_tb(etb),
+    message = paste0(traceback$format_exception(etype, e, etb),
+                     collapse = "")
+  )
+  out <- lapply(out, as_r_value)
+  attr(out, "exception") <- e
+  class(out) <- "py_error"
+  out
+}
+
+#' @export
+print.py_error <- function(x, ...) {
+  cat(x$message, "\n", sep = "")
 }
