@@ -1,6 +1,12 @@
 
 `%||%` <- function(x, y) if (is.null(x)) y else x
 
+
+safe_do_call <- function(fn, args) {
+  tryCatch(list(do.call(fn, args), FALSE),
+           error = function(e) list(e$message, TRUE))
+}
+
 traceback_enabled <- function() {
 
   # if there is specific option set then respect it
@@ -74,29 +80,6 @@ disable_conversion_scope <- function(object) {
   TRUE
 }
 
-new_stack <- function() {
-
-  (function() {
-
-    .data <- list()
-
-    methods <- list(
-      clear  = function() { .data <<- character() },
-      data   = function() { .data },
-      empty  = function() { length(.data) == 0 },
-      length = function() { length(.data) },
-      push   = function(line) { .data[[length(.data) + 1]] <<- line },
-      peek   = function() { .data[[length(.data)]] },
-      pop    = function() { .data <<- utils::head(.data, n = -1) },
-      set    = function(data) { .data <<- data }
-    )
-
-    list2env(methods)
-
-  })()
-
-}
-
 py_compile_eval <- function(code, compile_mode = "single", capture = TRUE) {
 
   builtins <- import_builtins(convert = TRUE)
@@ -104,8 +87,8 @@ py_compile_eval <- function(code, compile_mode = "single", capture = TRUE) {
 
   # allow 'globals' and 'locals' to both point at main module, so that
   # evaluated code updates references there as well
-  globals <- py_eval("globals()", convert = FALSE)
-  locals <- globals
+  main <- import_main(convert = FALSE)
+  globals <- locals <- py_get_attr(main, "__dict__")
 
   # Python's command compiler complains if the only thing you submit
   # is a comment, so detect that case first
@@ -138,9 +121,11 @@ py_compile_eval <- function(code, compile_mode = "single", capture = TRUE) {
 }
 
 py_last_value <- function() {
+  ex <- .globals$py_last_exception
+  on.exit(.globals$py_last_exception <- ex)
   tryCatch(
     py_eval("_", convert = FALSE),
-    error = function(e) r_to_py(NULL)
+    error = function(e) py_none()
   )
 }
 
@@ -197,8 +182,8 @@ file_same <- function(lhs, rhs) {
     return(TRUE)
 
   # check if file info is the same
-  lhsi <- c(file.info(lhs, extra_cols = FALSE))
-  rhsi <- c(file.info(rhs, extra_cols = FALSE))
+  lhsi <- suppressWarnings(c(file.info(lhs, extra_cols = FALSE)))
+  rhsi <- suppressWarnings(c(file.info(rhs, extra_cols = FALSE)))
   fields <- c("size", "isdir", "mode", "mtime", "ctime")
   if (identical(lhsi[fields], rhsi[fields]))
     return(TRUE)
@@ -232,7 +217,7 @@ enumerate <- function(x, f, ...) {
 }
 
 is_interactive <- function() {
-  
+
   # detect case where RStudio is being used, but reticulate is being
   # executed as part of a user's .Rprofile (for example). in this case,
   # we aren't really interactive as we are unable to respond to readline
@@ -241,25 +226,191 @@ is_interactive <- function() {
   gui <- .Platform$GUI
   if (rstudio && !identical(gui, "RStudio"))
     return(FALSE)
-  
+
   # otherwise, use base implementation
   interactive()
-  
+
 }
 
 is_r_cmd_check <- function() {
-  
+
   # if NOT_CRAN is set, this is likely devtools::check() -- allow it
   not_cran <- Sys.getenv("NOT_CRAN", unset = NA)
   if (identical(not_cran, "true"))
     return(FALSE)
-  
+
   # if _R_CHECK_PACKAGE_NAME_ is set, then we must be running R CMD check
   package_name <- Sys.getenv("_R_CHECK_PACKAGE_NAME_", unset = NA)
   if (!is.na(package_name))
     return(TRUE)
-  
+
   # does not appear to be R CMD check
   FALSE
-  
+
+}
+
+stack <- function(mode = "list") {
+
+  .data <- vector(mode)
+
+  object <- list(
+
+    set = function(data) {
+      .data <<- data
+    },
+
+    push = function(...) {
+      dots <- list(...)
+      for (data in dots) {
+        if (is.null(data))
+          .data[length(.data) + 1] <<- list(NULL)
+        else
+          .data[[length(.data) + 1]] <<- data
+      }
+    },
+
+    pop = function() {
+      item <- .data[[length(.data)]]
+      length(.data) <<- length(.data) - 1
+      item
+    },
+
+    peek = function() {
+      .data[[length(.data)]]
+    },
+
+    contains = function(data) {
+      data %in% .data
+    },
+
+    empty = function() {
+      length(.data) == 0
+    },
+
+    clear = function() {
+      .data <<- list()
+    },
+
+    data = function() {
+      .data
+    }
+
+  )
+
+  object
+
+}
+
+get_hooks_list <- function(name) {
+  hooks <- getHook(name)
+  if (!is.list(hooks))
+    hooks <- list(hooks)
+  hooks
+}
+
+deparse1 <- function(expr, width.cutoff = 500L) {
+  paste(deparse(expr, width.cutoff), collapse = " ")
+}
+
+isTRUE <- function(x) {
+  is.logical(x) && length(x) == 1L && !is.na(x) && x
+}
+
+isFALSE <- function(x) {
+  is.logical(x) && length(x) == 1L && !is.na(x) && !x
+}
+
+home <- function() {
+  path.expand("~")
+}
+
+aliased_path <- function(path) {
+
+  home <- home()
+  if (!nzchar(home))
+    return(path)
+
+  home <- chartr("\\", "/", home)
+  path <- chartr("\\", "/", path)
+
+  match <- regexpr(home, path, fixed = TRUE, useBytes = TRUE)
+  path[match == 1] <- file.path("~", substring(path[match == 1], nchar(home) + 2L))
+
+  path
+
+}
+
+pretty_path <- function(path) {
+  encodeString(aliased_path(path), quote = '"')
+}
+
+heredoc <- function(text) {
+
+  # remove leading, trailing whitespace
+  trimmed <- gsub("^\\s*\\n|\\n\\s*$", "", text)
+
+  # split into lines
+  lines <- strsplit(trimmed, "\n", fixed = TRUE)[[1L]]
+
+  # compute common indent
+  indent <- regexpr("[^[:space:]]", lines)
+  common <- min(setdiff(indent, -1L))
+
+  # remove common indent
+  paste(substring(lines, common), collapse = "\n")
+
+}
+
+dir.exists <- function(paths) {
+  utils::file_test("-d", paths)
+}
+
+str_split1_on_first <- function(x, pattern, ...) {
+  stopifnot(length(x) == 1, is.character(x))
+  regmatches(x, regexpr(pattern, x, ...), invert = TRUE)[[1L]]
+}
+
+str_drop_prefix <- function(x, prefix) {
+
+  if (is.character(prefix)) {
+    if (!startsWith(x, prefix))
+      return(x)
+    prefix <- nchar(prefix)
+  }
+
+  substr(x, as.integer(prefix) + 1L, nchar(x))
+
+}
+
+startsWith <- function(x, prefix) {
+  if (!is.character(x) || !is.character(prefix))
+    stop("non-character object(s)")
+  suppressWarnings(substr(x, 1L, nchar(prefix)) == prefix)
+}
+
+if (getRversion() < "3.3.0")
+endsWith <- function (x, suffix) { # needed for R < 3.3
+  if (!is.character(x) || !is.character(suffix))
+    stop("non-character object(s)")
+  n <- nchar(x)
+  suppressWarnings(substr(x, n - nchar(suffix) + 1L, n) == suffix)
+}
+
+debuglog <- function(fmt, ...) {
+  msg <- sprintf(fmt, ...)
+  cat(msg, file = "/tmp/reticulate.log", sep = "\n", append = TRUE)
+}
+
+system2t <- function(command, args, ...) {
+  # system2, with a trace
+  # mimic bash's set -x usage of a "+" prefix for now
+  # maybe someday take a dep on {cli} and make it prettier
+  message(paste("+", shQuote(command), paste0(args, collapse = " ")))
+  system2(command, args, ...)
+}
+
+
+rm_all_reticulate_state <- function() {
+  unlink(rappdirs::user_data_dir("r-reticulate", NULL), recursive = TRUE, force = TRUE)
+  unlink(rappdirs::user_data_dir("r-miniconda", NULL), recursive = TRUE, force = TRUE)
 }
